/*
    Bantumi
    Copyright 2011 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#ifndef __JNIUTILS_H
#define __JNIUTILS_H

#include <jni.h>

void debugPrintf(JNIEnv* env, const char* fmt, ...);

template<class T> T getPrivData(JNIEnv* env, jobject thiz) {
	jclass clazz = env->GetObjectClass(thiz);
	jfieldID privField = env->GetFieldID(clazz, "mPrivData", "J");
	jlong ptr = env->GetLongField(thiz, privField);
	T value = (T) ptr;
	env->DeleteLocalRef(clazz);
	return value;
}

template<class T> void setPrivData(JNIEnv* env, jobject thiz, T value) {
	jclass clazz = env->GetObjectClass(thiz);
	jfieldID privField = env->GetFieldID(clazz, "mPrivData", "J");
	jlong ptr = (jlong) value;
	env->SetLongField(thiz, privField, ptr);
	env->DeleteLocalRef(clazz);
}

void throwException(JNIEnv* env, const char* classname, const char* message);
bool checkNull(JNIEnv* env, void* ptr);

#endif

