/*
    Bantumi
    Copyright 2011 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#include <stdio.h>
#include "jniutils.h"

void debugPrintf(JNIEnv* env, const char* fmt, ...) {
	va_list ap;
	va_start(ap, fmt);
	char buf[1000];
	vsnprintf(buf, sizeof(buf), fmt, ap);
	jstring str = env->NewStringUTF(buf);
	jclass system = env->FindClass("java/lang/System");
	jobject out = env->GetStaticObjectField(system, env->GetStaticFieldID(system, "out", "Ljava/io/PrintStream;"));
	jclass outclass = env->GetObjectClass(out);
	env->CallVoidMethod(out, env->GetMethodID(outclass, "println", "(Ljava/lang/String;)V"), str);
	env->DeleteLocalRef(outclass);
	env->DeleteLocalRef(out);
	env->DeleteLocalRef(system);
	env->DeleteLocalRef(str);
	va_end(ap);
}


void throwException(JNIEnv* env, const char* classname, const char* message) {
	jclass clazz = env->FindClass(classname);
	env->ThrowNew(clazz, message);
	env->DeleteLocalRef(clazz);
}

bool checkNull(JNIEnv* env, void* ptr) {
	if (ptr)
		return false;
	throwException(env, "java/lang/NullPointerException", "Private pointer is null");
	return true;
}

