/*
    Bantumi
    Copyright 2011 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

package st.martin.bantumi;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Field;
import java.nio.ByteBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

public class BantumiRenderer implements android.opengl.GLSurfaceView.Renderer {

	public BantumiRenderer(BantumiActivity ba) {
		mActivity = ba;
	}

	public void onDrawFrame(GL10 gl) {
		nativeOnDrawFrame(mActivity, System.currentTimeMillis());
		if (mActivity.shouldExit())
			mActivity.finish();
	}

	public void onSurfaceChanged(GL10 gl, int width, int height) {
		nativeOnSurfaceChanged(mActivity, width, height);
		createFonts();
	}

	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		nativeOnSurfaceCreated(mActivity);
	}
	
	public void createFonts() {
		createFont(false, R.drawable.numfont, R.raw.numfont);
		createFont(true, R.drawable.textfont, R.raw.textfont);
	}

	public void createFont(boolean text, int bitmapId, int infoId) {
		BitmapFactory.Options options = new BitmapFactory.Options();
		try {
			Class<?> c = options.getClass();
			Field f = c.getField("inScaled");
			f.setBoolean(options, false);
		} catch (NoSuchFieldException e) {
		} catch (IllegalAccessException e) {
		}
		Bitmap bitmap = BitmapFactory.decodeResource(mActivity.getResources(), bitmapId, options).copy(Bitmap.Config.ARGB_8888, false);
		BufferedReader infoReader = new BufferedReader(new InputStreamReader(mActivity.getResources().openRawResource(infoId)));
		try {
			String info = infoReader.readLine();
			String[] infoFields = info.split(" ");
			int offset = Integer.parseInt(infoFields[0]);
			int n = Integer.parseInt(infoFields[1]);
			int charInfo[] = new int[8*n];
			int infoPos = 0;
			while ((info = infoReader.readLine()) != null) {
				infoFields = info.split(" ");
				for (int i = 0; i < 8; i++)
					charInfo[infoPos + i] = Integer.parseInt(infoFields[i]);
				infoPos += 8;
			}
			ByteBuffer buffer = ByteBuffer.allocate(4*bitmap.getWidth()*bitmap.getHeight());
			bitmap.copyPixelsToBuffer(buffer);
			setFont(mActivity, text, buffer.array(), false, bitmap.getWidth(), bitmap.getHeight(), charInfo, offset, n, infoPos/8);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	private native void nativeOnDrawFrame(Object activity, long time);
	private native void nativeOnSurfaceCreated(Object activity);
	private native void nativeOnSurfaceChanged(Object activity, int width, int height);
	private native void setFont(Object activity, boolean text, byte[] image, boolean alphaBitmap, int w, int h, int[] info, int offset, int n, int chars);
	
	private BantumiActivity mActivity;
}
