/*
    Bantumi
    Copyright 2011 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

package st.martin.bantumi;

import java.io.IOException;
import java.util.UUID;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.content.Intent;

public class BluetoothManager {
	private static final UUID mServiceUuid = UUID.fromString("10273929-0000-1000-8000-00805F9B34FB");

	BluetoothAdapter mAdapter = BluetoothAdapter.getDefaultAdapter();

	private static final int REQUEST_ENABLE_BT = 1;
	private static final int REQUEST_ENABLE_DISCOVERABLE = 2;

	private Runnable mRunWhenBluetoothEnabled;
	private Runnable mRunWhenBluetoothEnabledFailed;
	private Runnable mRunWhenBluetoothDiscoverableFailed;
	public void turnOnBluetooth(Runnable ok, Runnable failed, Activity activity) {
		if (mAdapter == null) {
			failed.run();
			return;
		}
		if (!mAdapter.isEnabled()) {
			Intent intent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
			activity.startActivityForResult(intent, REQUEST_ENABLE_BT);
			mRunWhenBluetoothEnabled = ok;
			mRunWhenBluetoothEnabledFailed = failed;
			return;
		}
		ok.run();
	}
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
		case REQUEST_ENABLE_BT:
			if (resultCode == Activity.RESULT_CANCELED) {
				if (mRunWhenBluetoothEnabledFailed != null)
					mRunWhenBluetoothEnabledFailed.run();
				mRunWhenBluetoothEnabledFailed = null;
			} else if (resultCode == Activity.RESULT_OK) {
				if (mRunWhenBluetoothEnabled != null)
					mRunWhenBluetoothEnabled.run();
				mRunWhenBluetoothEnabled = null;
			}
			break;
		case REQUEST_ENABLE_DISCOVERABLE:
			if (resultCode == Activity.RESULT_CANCELED) {
				if (mRunWhenBluetoothDiscoverableFailed != null)
					mRunWhenBluetoothDiscoverableFailed.run();
				mRunWhenBluetoothDiscoverableFailed = null;
			}
			break;
		}
	}
	public ConnectionThread doBluetoothHost(Runnable failed, Activity activity, ConnectionManager manager) throws IOException {
		mRunWhenBluetoothDiscoverableFailed = failed;
		BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
		Intent intent = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
		intent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
		activity.startActivityForResult(intent, REQUEST_ENABLE_DISCOVERABLE);
		BluetoothServerSocket socket = adapter.listenUsingRfcommWithServiceRecord("Bantumi", mServiceUuid);
		return new ConnectionThread(manager, activity, socket);
	}
	public ConnectionThread connectTo(BluetoothDevice dev, ConnectionManager manager, Context context) throws IOException {
		BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
		adapter.cancelDiscovery();
		BluetoothSocket socket = dev.createRfcommSocketToServiceRecord(mServiceUuid);
		return new ConnectionThread(manager, context, socket);
	}

}
